
/* 
 *                  here was the
 *  ______________    connector   ______________
 * |            o |______________| o            |
 * |               | |  ||||||||      ####      |
 * o>              + G  nSnTRnnn      chip     <o
 * |               3 N  c c  ccc      ####      |
 * |                 D                          |
 * | o                                        o |
 *  \__________________________________________/
 * 
 * place some resistor (100 ohm in my circuit) between +3V and S
 * 	(which appears to be 'keyboard select')
 * 
 * connect TX from Agenda to T, RX from Agenda to R.
 */

#include "standard.h"

extern byte chatboard_cnt, chatboard_buf[];

void
chatboard_debug()
{
	static byte cb_rem, fill;

	if (cb_rem != chatboard_cnt) {
		print_string("cb ");

		while (cb_rem != chatboard_cnt) {
			vis(chatboard_buf + cb_rem, 1);
			cb_rem++;
		}
		eol();
	}
}

static int
characterize()
{
	byte c, past;

#define prefix(x) !bcmp(chatboard_buf, (x), past = sizeof(x) - 1)

	if (prefix("AT*EKSE=")) {
		/*
		 *                0-9   AT*EKSE=0    ...=9
		 *        shift   0-9   AT*EKSE=32   ...=41
		 *                a-z   AT*EKSE=65   ...=90
		 *                spac  AT*EKSE=91
		 *                .     AT*EKSE=92
		 *  sh-spac       a-z   AT*EKSE=97   ...=122
		 *  sh-spac       spac  AT*EKSE=123
		 *  sh-spac       .     AT*EKSE=124
		 *          shift a-z   AT*EKSE=129  ...=154
		 *          shift .     AT*EKSE=156
		 *  sh-spac shift a-z   AT*EKSE=161  ...=186
		 *  sh-spac shift .     AT*EKSE=188
		 */
		c = atoi(&chatboard_buf[past]);

		if (c < 10)
			return ('0' + c);

		if (c < 91)
			return ('a' + c - 65);
		if (c == 91)
			return (' ');
		if (c == 92)
			return ('.');

		goto dunno;
	}
	if (prefix("AT+CKPD=\"")) {
		/*
		 *               Attach AT+CKPD="#:"
		 *               Backsp AT+CKPD="C"        BS
		 *               NO     AT+CKPD="E"        ESC
		 *               YES    AT+CKPD="S"        CR
		 *               <-     AT+CKPD="<"
		 *               ->     AT+CKPD=">" 
		 */
		c = chatboard_buf[past] | 0x20; /* force CES to ces */

		if (c == 'c')
			return (010);
		if (c == 'e')
			return (033);
		if (c == 's')
			return ('\r');

		goto dunno;
	}
	if (prefix("AT*EAPP=")) {
		/*
		 *               E-mail AT*EAPP=0,5,"To: ","WWW"
		 *               WWW    AT*EAPP=0,5,"WWW: ","WWW"
		 *               SMS    AT*EAPP=0,0
		 *               PhBook AT*EAPP=1,0
		 */
		c = chatboard_buf[past];

		goto dunno;
	}
		/*
		 *  Power on
		 *   AT           aaaaalo ?
		 *   AT+CGMM      request model identification
		 *   AT+CGMR      request revision identification
		 *  WWW key
		 *   AT*EDME?     are data menus enabled?
		 *   AT*ESVM?     current voice mail server number?
		 *   AT+CKPD="eseee<see<<s>>ss1"
		 *   AT+CKPD="c",25
		 *   AT+CKPD="91c91c911111110c"
		 *  Email key
		 *   AT*EDME?     are data menus enabled?
		 *   AT*ESVM?     current voice mail server number?
		 *   AT+CKPD="eseee<see<<s>>ss1"
		 *   AT+CKPD="c",25
		 *   AT+CKPD="8666*1111111*0c"
		 *  SMS key
		 *   AT*EDME?     are data menus enabled?
		 *   AT*ESVM?     current voice mail server number?
		 *   AT+CKPD="eseee<see<<s>>ss1"
		 *   AT+CKPD="c",25
		 *  Phonebook key
		 *   AT*EDME?     are data menus enabled?
		 *   AT*ESVM?     current voice mail server number?
		 *   AT+CKPD="eseee<see<<s>>s"
		 *  Attachment key
		 *   AT+CKPD="#11111110c"
		 *  Yes key
		 *   AT+CKPD="s"
		 *  No key
		 *   AT+CKPD="e"
		 *  Inside SMS
		 *     AT+CKPD="1",20         key 1
		 *     AT+CKPD="2",20         key 2
		 *     AT+CKPD="2"            key A
		 *     AT+CKPD="22"           key B     etc
		 */
dunno:
	return (0);
}

/* chatboard_init(); needs to be called once */

int
chatboard_getc()
{
	int  c = 0;
	word n = chatboard_cnt;

	if (n && chatboard_buf[--n] == '\r') {

		chatboard_buf[n] = 0;
		c = characterize();

		chatboard_cnt = 0;
		chatboard_OK();
	}
	return (c);
}

