
W_RESET equ 0000h ; kzsp pc sp iz

V_BKPT   equ 0008h
V_BERR   equ 000Ah
V_SWI    equ 000Ch
V_ILL    equ 000Eh
V_ZDIV   equ 0010h
V_UNINIT equ 001Eh
V_L1AV   equ 0022h
V_L2AV   equ 0024h
V_L3AV   equ 0026h
V_L4AV   equ 0028h
V_L5AV   equ 002Ah
V_L6AV   equ 002Ch
V_L7AV   equ 002Eh
V_SPUR   equ 0030h


	;------------------------------------
	section .rodata
	align 1

msg_exception   dc.b 'UNKNOWN EXCEPTION'
                dc.b 0
msg_interrupt   dc.b 'UNKNOWN INTERRUPT'
                dc.b 0
msg_bkpt        dc.b 'BREAKPOINT'
                dc.b 0
msg_berr        dc.b 'BUS ERROR'
                dc.b 0
msg_swi         dc.b 'SOFTWARE INTERRUPT'
                dc.b 0
msg_ill         dc.b 'ILLEGAL INSTRUCTION'
                dc.b 0
msg_zdiv        dc.b 'DIVISION BY ZERO'
                dc.b 0
msg_uninit      dc.b 'UNINITIALIZED INTERRUPT'
				dc.b 0
msg_l1av        dc.b 'LEVEL1 AVEC'
                dc.b 0
msg_l2av        dc.b 'LEVEL2 AVEC'
                dc.b 0
msg_l3av        dc.b 'LEVEL3 AVEC'
                dc.b 0
msg_l4av        dc.b 'LEVEL4 AVEC'
                dc.b 0
msg_l5av        dc.b 'LEVEL5 AVEC'
                dc.b 0
msg_l6av        dc.b 'LEVEL6 AVEC'
                dc.b 0
msg_l7av        dc.b 'LEVEL7 AVEC'
                dc.b 0
msg_spur        dc.b 'SPURIOUS INTERRUPT'
                dc.b 0

	;------------------------------------
	section .inthand
	align 1

trap_exception
	ldx  # msg_exception
	jmp  panic
trap_interrupt
	ldx  # msg_interrupt
	jmp  panic

trap_bkpt
	ldx  # msg_bkpt
	jmp  panic
trap_berr
	ldx  # msg_berr
	jmp  panic
trap_swi
	ldx  # msg_swi
	jmp  panic
trap_ill
	ldx  # msg_ill
	jmp  panic
trap_zdiv
	ldx  # msg_zdiv
	jmp  panic
trap_uninit
	ldx  # msg_uninit
	jmp  panic
trap_l1av
	ldx  # msg_l1av
	jmp  panic
trap_l2av
	ldx  # msg_l2av
	jmp  panic
trap_l3av
	ldx  # msg_l3av
	jmp  panic
trap_l4av
	ldx  # msg_l4av
	jmp  panic
trap_l5av
	ldx  # msg_l5av
	jmp  panic
trap_l6av
	ldx  # msg_l6av
	jmp  panic
trap_l7av
	ldx  # msg_l7av
	jmp  panic
trap_spur
	ldx  # msg_spur
	jmp  panic

	;------------------------------------
	section .text
	align 1

	global _default_exception_vectors

_default_exception_vectors

	pshm k,ccr
	orp  # 00E0h

	clrb
	tbek
	tbxk

	ldx  # 0008h
	ldd  # trap_exception
L1
	std  0,x
	aix  # 2
	cpx  # 0070h
	bne  L1
	ldd  # trap_interrupt
L2
	std  0,x
	aix  # 2
	cpx  # 0200h
	bne  L2

	ldd  # trap_bkpt
	std  V_BKPT
	ldd  # trap_berr
	std  V_BERR
	ldd  # trap_swi
	std  V_SWI
	ldd  # trap_ill
	std  V_ILL
	ldd  # trap_zdiv
	std  V_ZDIV
	ldd  # trap_uninit
	std  V_UNINIT

	ldd  # trap_l1av
	std  V_L1AV
	ldd  # trap_l2av
	std  V_L2AV
	ldd  # trap_l3av
	std  V_L3AV
	ldd  # trap_l4av
	std  V_L4AV
	ldd  # trap_l5av
	std  V_L5AV
	ldd  # trap_l6av
	std  V_L6AV
	ldd  # trap_l7av
	std  V_L7AV

	ldd  # trap_spur
	std  V_SPUR

	pulm k,ccr
	rts

SCCR1  equ 0FC0Ah
TE     equ 0008h   ; transmitter enable
RE     equ 0004h   ; transmitter enable
SCSR_  equ 0FC0Ch  ; MSByte only in mask
TDRE_  equ 01h     ; transmit data register empty
SCDR   equ 0FC0Fh  ; LSByte only

cputb	MACRO
	brclr SCSR_, # TDRE_, *    ; XXX can get stuck if brg has been fouled
	stab  SCDR
	ENDM

cputbh	MACRO
	andb # 15
	cmpb # 10
	bcs  * + 4
	addb # 7
	addb # '0'
	cputb
	ENDM

panic
	orp  # 00E0h

	ldd  # __text_page
	tbxk
	ldab # 0FFh
	tbek

	ldd  # TE + RE
	std  SCCR1

	ldab # 0Ah
	cputb
	ldab # 0Dh
	cputb
pnc2
	ldab 0,x
	beq  pnc3
	cputb
	aix  # 1
	bra  pnc2
pnc3
	ldab # 20h
	cputb

	pulm d
	cputbh

	pulm e
	ted
	xgab
	lsrb
	lsrb
	lsrb
	lsrb
	cputbh
	ted
	xgab
	cputbh
	ted
	lsrb
	lsrb
	lsrb
	lsrb
	cputbh
	ted
	cputbh

	global _reboot

_reboot

	orp  # 00E0h

	clrb
	tbek
	ldx  2             ; ignore rest of reset values
	jmp  20000h, x

